@@ -57,7 +57,6 @@
 				greynetic -root				    \\n\
 				helix -root				    \\n\
 				hopalong -root				    \\n\
--				hypercube -root				    \\n\
 				imsmap -root				    \\n\
 -				noseguy -root				    \\n\
 -				pyro -root				    \\n\
@@ -141,7 +140,6 @@
 				xflame -root				    \\n\
 				xmatrix -root				    \\n\
   GL: 				gflux -root				    \\n\
--				hyperball -root				    \\n\
 -				nerverot -root				    \\n\
 				xrayswarm -root				    \\n\
 				xspirograph -root			    \\n\
@@ -171,7 +169,7 @@
 - GL: 				glblur -root				    \\n\
   GL: 				glsnake -root				    \\n\
 				halftone -root				    \\n\
--				juggle -root				    \\n\
+  GL: 				juggler3d -root				    \\n\
   GL: 				lavalite -root				    \\n\
 -				polyominoes -root			    \\n\
   GL: 				queens -root				    \\n\
@@ -198,13 +196,13 @@
 - GL: 				glslideshow -root			    \\n\
   GL: 				hypertorus -root			    \\n\
 - GL: 				jigglypuff -root			    \\n\
-  GL: 				klein -root				    \\n\
 				metaballs -root				    \\n\
   GL: 				mirrorblob -root			    \\n\
 				piecewise -root				    \\n\
   GL: 				polytopes -root				    \\n\
 				pong -root				    \\n\
 				popsquares -root			    \\n\
+  GL: 				surfaces -root				    \\n\
 				xanalogtv -root				    \\n\
 -				abstractile -root			    \\n\
 				anemotaxis -root			    \\n\
@@ -231,7 +229,6 @@
 				fiberlamp -root				    \\n\
   GL: 				fliptext -root				    \\n\
   GL: 				glhanoi -root				    \\n\
-  GL: 				juggler3d -root				    \\n\
   GL: 				tangram -root				    \\n\
   GL: 				timetunnel -root			    \\n\
   GL: 				glschool -root				    \\n\
@@ -245,9 +242,11 @@
   GL: 				moebiusgears -root			    \\n\
   GL: 				voronoi -root				    \\n\
   GL: 				hypnowheel -root			    \\n\
+  GL: 				klein -root				    \\n\
 -				lcdscrub -root				    \\n\
   GL: 				photopile -root				    \\n\
-  GL: 				skytentacles -root			    \\n",
+  GL: 				skytentacles -root			    \\n\
+  GL: 				rubikblocks -root			    \\n",
 "XScreenSaver.pointerPollTime:		0:00:05",
 "XScreenSaver.pointerHysteresis:		10",
 "XScreenSaver.initialDelay:		0:00:00",
@@ -338,6 +337,7 @@
 "*hacks.rd-bomb.name:        RDbomb",
 "*hacks.rdbomb.name:         RDbomb",
 "*hacks.rotzoomer.name:      RotZoomer",
+"*hacks.rubikblocks.name:    RubikBlocks",
 "*hacks.sballs.name:         SBalls",
 "*hacks.shadebobs.name:      ShadeBobs",
 "*hacks.sierpinski3d.name:   Sierpinski3D",
